package com.link.core.xstream;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.converters.ConversionException;
import com.thoughtworks.xstream.converters.Converter;
import com.thoughtworks.xstream.converters.ConverterMatcher;
import com.thoughtworks.xstream.converters.SingleValueConverter;
import com.thoughtworks.xstream.io.HierarchicalStreamDriver;
import com.thoughtworks.xstream.io.HierarchicalStreamReader;
import com.thoughtworks.xstream.io.HierarchicalStreamWriter;
import com.thoughtworks.xstream.io.StreamException;
import com.thoughtworks.xstream.io.xml.CompactWriter;
import com.thoughtworks.xstream.io.xml.DomReader;
import com.thoughtworks.xstream.io.xml.DomWriter;
import com.thoughtworks.xstream.io.xml.QNameMap;
import com.thoughtworks.xstream.io.xml.SaxWriter;
import com.thoughtworks.xstream.io.xml.StaxReader;
import com.thoughtworks.xstream.io.xml.StaxWriter;
import com.thoughtworks.xstream.io.xml.XmlFriendlyReplacer;
import com.thoughtworks.xstream.io.xml.XppReader;
import com.thoughtworks.xstream.mapper.CannotResolveClassException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.Writer;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLEventWriter;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamWriter;
import org.springframework.beans.factory.BeanClassLoaderAware;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.oxm.MarshallingFailureException;
import org.springframework.oxm.UncategorizedMappingException;
import org.springframework.oxm.UnmarshallingFailureException;
import org.springframework.oxm.XmlMappingException;
import org.springframework.oxm.support.AbstractMarshaller;
import org.springframework.util.Assert;
import org.springframework.util.ClassUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.util.xml.StaxUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.ext.LexicalHandler;

public class MyXStreamMarshaller extends AbstractMarshaller implements InitializingBean, BeanClassLoaderAware {
   public static final String DEFAULT_ENCODING = "UTF-8";
   private final XStream xstream = new XStream();
   private HierarchicalStreamDriver streamDriver;
   private String encoding = "UTF-8";
   private Class<?>[] supportedClasses;
   private ClassLoader beanClassLoader;

   public MyXStreamMarshaller() {
      this.xstream.registerConverter(new DataTypeConverter());
   }

   public XStream getXStream() {
      return this.xstream;
   }

   public void setMode(int mode) {
      this.getXStream().setMode(mode);
   }

   public void setConverters(ConverterMatcher... converters) {
      for(int i = 0; i < converters.length; ++i) {
         if (converters[i] instanceof Converter) {
            this.getXStream().registerConverter((Converter)converters[i], i);
         } else {
            if (!(converters[i] instanceof SingleValueConverter)) {
               throw new IllegalArgumentException("Invalid ConverterMatcher [" + converters[i] + "]");
            }

            this.getXStream().registerConverter((SingleValueConverter)converters[i], i);
         }
      }

   }

   public void setAliases(Map<String, ?> aliases) throws ClassNotFoundException {
      Map<String, Class<?>> classMap = this.toClassMap(aliases);
      Iterator var3 = classMap.entrySet().iterator();

      while(var3.hasNext()) {
         Map.Entry<String, Class<?>> entry = (Map.Entry)var3.next();
         this.getXStream().alias((String)entry.getKey(), (Class)entry.getValue());
      }

   }

   public void setAliasesByType(Map<String, ?> aliases) throws ClassNotFoundException {
      Map<String, Class<?>> classMap = this.toClassMap(aliases);
      Iterator var3 = classMap.entrySet().iterator();

      while(var3.hasNext()) {
         Map.Entry<String, Class<?>> entry = (Map.Entry)var3.next();
         this.getXStream().aliasType((String)entry.getKey(), (Class)entry.getValue());
      }

   }

   private Map<String, Class<?>> toClassMap(Map<String, ?> map) throws ClassNotFoundException {
      Map<String, Class<?>> result = new LinkedHashMap(map.size());

      String key;
      Class type;
      for(Iterator var3 = map.entrySet().iterator(); var3.hasNext(); result.put(key, type)) {
         Map.Entry<String, ?> entry = (Map.Entry)var3.next();
         key = (String)entry.getKey();
         Object value = entry.getValue();
         if (value instanceof Class) {
            type = (Class)value;
         } else {
            if (!(value instanceof String)) {
               throw new IllegalArgumentException("Unknown value [" + value + "] - expected String or Class");
            }

            String className = (String)value;
            type = ClassUtils.forName(className, this.beanClassLoader);
         }
      }

      return result;
   }

   public void setFieldAliases(Map<String, String> aliases) throws ClassNotFoundException, NoSuchFieldException {
      Iterator var2 = aliases.entrySet().iterator();

      while(var2.hasNext()) {
         Map.Entry<String, String> entry = (Map.Entry)var2.next();
         String alias = (String)entry.getValue();
         String field = (String)entry.getKey();
         int idx = field.lastIndexOf(46);
         if (idx == -1) {
            throw new IllegalArgumentException("Field name [" + field + "] does not contain '.'");
         }

         String className = field.substring(0, idx);
         Class<?> clazz = ClassUtils.forName(className, this.beanClassLoader);
         String fieldName = field.substring(idx + 1);
         this.getXStream().aliasField(alias, clazz, fieldName);
      }

   }

   public void setUseAttributeForTypes(Class<?>... types) {
      Class[] var2 = types;
      int var3 = types.length;

      for(int var4 = 0; var4 < var3; ++var4) {
         Class<?> type = var2[var4];
         this.getXStream().useAttributeFor(type);
      }

   }

   public void setUseAttributeFor(Map<?, ?> attributes) {
      Iterator var2 = attributes.entrySet().iterator();

      while(true) {
         while(var2.hasNext()) {
            Map.Entry<?, ?> entry = (Map.Entry)var2.next();
            if (entry.getKey() instanceof String) {
               if (!(entry.getValue() instanceof Class)) {
                  throw new IllegalArgumentException("'useAttributesFor' takes Map<String, Class> when using a map key of type String");
               }

               this.getXStream().useAttributeFor((String)entry.getKey(), (Class)entry.getValue());
            } else {
               if (!(entry.getKey() instanceof Class)) {
                  throw new IllegalArgumentException("'useAttributesFor' property takes either a map key of type String or Class");
               }

               Class<?> key = (Class)entry.getKey();
               if (entry.getValue() instanceof String) {
                  this.getXStream().useAttributeFor(key, (String)entry.getValue());
               } else {
                  if (!(entry.getValue() instanceof List)) {
                     throw new IllegalArgumentException("'useAttributesFor' property takes either Map<Class, String> or Map<Class, List<String>> when using a map key of type Class");
                  }

                  List listValue = (List)entry.getValue();
                  Iterator var6 = listValue.iterator();

                  while(var6.hasNext()) {
                     Object element = var6.next();
                     if (element instanceof String) {
                        this.getXStream().useAttributeFor(key, (String)element);
                     }
                  }
               }
            }
         }

         return;
      }
   }

   public void setImplicitCollections(Map<Class<?>, String> implicitCollections) {
      Iterator var2 = implicitCollections.entrySet().iterator();

      while(var2.hasNext()) {
         Map.Entry<Class<?>, String> entry = (Map.Entry)var2.next();
         String[] collectionFields = StringUtils.commaDelimitedListToStringArray((String)entry.getValue());
         String[] var5 = collectionFields;
         int var6 = collectionFields.length;

         for(int var7 = 0; var7 < var6; ++var7) {
            String collectionField = var5[var7];
            this.getXStream().addImplicitCollection((Class)entry.getKey(), collectionField);
         }
      }

   }

   public void setOmittedFields(Map<Class<?>, String> omittedFields) {
      Iterator var2 = omittedFields.entrySet().iterator();

      while(var2.hasNext()) {
         Map.Entry<Class<?>, String> entry = (Map.Entry)var2.next();
         String[] fields = StringUtils.commaDelimitedListToStringArray((String)entry.getValue());
         String[] var5 = fields;
         int var6 = fields.length;

         for(int var7 = 0; var7 < var6; ++var7) {
            String field = var5[var7];
            this.getXStream().omitField((Class)entry.getKey(), field);
         }
      }

   }

   /** @deprecated */
   @Deprecated
   public void setAnnotatedClass(Class<?> annotatedClass) {
      Assert.notNull(annotatedClass, "'annotatedClass' must not be null");
      this.getXStream().processAnnotations(annotatedClass);
   }

   public void setAnnotatedClasses(Class<?>... annotatedClasses) {
      Assert.notEmpty(annotatedClasses, "'annotatedClasses' must not be empty");
      this.getXStream().processAnnotations(annotatedClasses);
   }

   public void setAutodetectAnnotations(boolean autodetectAnnotations) {
      this.getXStream().autodetectAnnotations(autodetectAnnotations);
   }

   public void setStreamDriver(HierarchicalStreamDriver streamDriver) {
      this.streamDriver = streamDriver;
   }

   public void setEncoding(String encoding) {
      this.encoding = encoding;
   }

   public void setSupportedClasses(Class<?>... supportedClasses) {
      this.supportedClasses = supportedClasses;
   }

   public void setBeanClassLoader(ClassLoader classLoader) {
      this.beanClassLoader = classLoader;
      this.getXStream().setClassLoader(classLoader);
   }

   public final void afterPropertiesSet() throws Exception {
      this.customizeXStream(this.getXStream());
   }

   protected void customizeXStream(XStream xstream) {
   }

   public boolean supports(Class<?> clazz) {
      if (ObjectUtils.isEmpty(this.supportedClasses)) {
         return true;
      } else {
         Class[] var2 = this.supportedClasses;
         int var3 = var2.length;

         for(int var4 = 0; var4 < var3; ++var4) {
            Class<?> supportedClass = var2[var4];
            if (supportedClass.isAssignableFrom(clazz)) {
               return true;
            }
         }

         return false;
      }
   }

   protected void marshalDomNode(Object graph, Node node) throws XmlMappingException {
      DomWriter streamWriter;
      if (node instanceof Document) {
         streamWriter = new DomWriter((Document)node);
      } else {
         if (!(node instanceof Element)) {
            throw new IllegalArgumentException("DOMResult contains neither Document nor Element");
         }

         streamWriter = new DomWriter((Element)node, node.getOwnerDocument(), new XmlFriendlyReplacer());
      }

      this.marshal(graph, streamWriter);
   }

   protected void marshalXmlEventWriter(Object graph, XMLEventWriter eventWriter) throws XmlMappingException {
      ContentHandler contentHandler = StaxUtils.createContentHandler(eventWriter);
      this.marshalSaxHandlers(graph, contentHandler, (LexicalHandler)null);
   }

   protected void marshalXmlStreamWriter(Object graph, XMLStreamWriter streamWriter) throws XmlMappingException {
      try {
         this.marshal(graph, new StaxWriter(new QNameMap(), streamWriter));
      } catch (XMLStreamException var4) {
         XMLStreamException ex = var4;
         throw this.convertXStreamException(ex, true);
      }
   }

   protected void marshalOutputStream(Object graph, OutputStream outputStream) throws XmlMappingException, IOException {
      if (this.streamDriver != null) {
         this.marshal(graph, this.streamDriver.createWriter(outputStream));
      } else {
         this.marshalWriter(graph, new OutputStreamWriter(outputStream, this.encoding));
      }

   }

   protected void marshalSaxHandlers(Object graph, ContentHandler contentHandler, LexicalHandler lexicalHandler) throws XmlMappingException {
      SaxWriter saxWriter = new SaxWriter();
      saxWriter.setContentHandler(contentHandler);
      this.marshal(graph, saxWriter);
   }

   protected void marshalWriter(Object graph, Writer writer) throws XmlMappingException, IOException {
      if (this.streamDriver != null) {
         this.marshal(graph, this.streamDriver.createWriter(writer));
      } else {
         this.marshal(graph, new CompactWriter(writer));
      }

   }

   private void marshal(Object graph, HierarchicalStreamWriter streamWriter) {
      try {
         this.getXStream().marshal(graph, streamWriter);
      } catch (Exception var11) {
         Exception ex = var11;
         throw this.convertXStreamException(ex, true);
      } finally {
         try {
            streamWriter.flush();
         } catch (Exception var10) {
            Exception ex = var10;
            this.logger.debug("Could not flush HierarchicalStreamWriter", ex);
         }

      }

   }

   protected Object unmarshalDomNode(Node node) throws XmlMappingException {
      DomReader streamReader;
      if (node instanceof Document) {
         streamReader = new DomReader((Document)node);
      } else {
         if (!(node instanceof Element)) {
            throw new IllegalArgumentException("DOMSource contains neither Document nor Element");
         }

         streamReader = new DomReader((Element)node);
      }

      return this.unmarshal(streamReader);
   }

   protected Object unmarshalXmlEventReader(XMLEventReader eventReader) throws XmlMappingException {
      try {
         XMLStreamReader streamReader = StaxUtils.createEventStreamReader(eventReader);
         return this.unmarshalXmlStreamReader(streamReader);
      } catch (XMLStreamException var3) {
         XMLStreamException ex = var3;
         throw this.convertXStreamException(ex, false);
      }
   }

   protected Object unmarshalXmlStreamReader(XMLStreamReader streamReader) throws XmlMappingException {
      return this.unmarshal(new StaxReader(new QNameMap(), streamReader));
   }

   protected Object unmarshalInputStream(InputStream inputStream) throws XmlMappingException, IOException {
      return this.streamDriver != null ? this.unmarshal(this.streamDriver.createReader(inputStream)) : this.unmarshalReader(new InputStreamReader(inputStream, this.encoding));
   }

   protected Object unmarshalReader(Reader reader) throws XmlMappingException, IOException {
      return this.streamDriver != null ? this.unmarshal(this.streamDriver.createReader(reader)) : this.unmarshal(new XppReader(reader));
   }

   protected Object unmarshalSaxReader(XMLReader xmlReader, InputSource inputSource) throws XmlMappingException, IOException {
      throw new UnsupportedOperationException("XStreamMarshaller does not support unmarshalling using SAX XMLReaders");
   }

   private Object unmarshal(HierarchicalStreamReader streamReader) {
      try {
         return this.getXStream().unmarshal(streamReader);
      } catch (Exception var3) {
         Exception ex = var3;
         throw this.convertXStreamException(ex, false);
      }
   }

   protected XmlMappingException convertXStreamException(Exception ex, boolean marshalling) {
      if (!(ex instanceof StreamException) && !(ex instanceof CannotResolveClassException) && !(ex instanceof ConversionException)) {
         return new UncategorizedMappingException("Unknown XStream exception", ex);
      } else {
         return (XmlMappingException)(marshalling ? new MarshallingFailureException("XStream marshalling exception", ex) : new UnmarshallingFailureException("XStream unmarshalling exception", ex));
      }
   }
}
